#!/usr/bin/perl

package Cognition::Export;
require Exporter;
@ISA       = qw(Exporter);
@EXPORT    = qw(allOf allOfObj firstOf inProp findSubject);

sub firstOf
{
	my $subject  = shift;
	return unless ($subject);
	foreach my $p (@_)
	{
		return $subject->firstVal($p)->toString
			if ($subject->property($p));
	}
	return;
}

sub allOf
{
	my $subject  = shift;
	return unless ($subject);
	my %rv;
	foreach my $p (@_)
	{
		next unless (defined $subject->property($p));
		foreach my $v ( @{$subject->property($p)->valObjects} )
		{
			$rv{ $v->toString } = 1;
		}
	}
	return (keys %rv);
}

sub allOfObj
{
	my $subject  = shift;
	return unless ($subject);
	my @rv;
	foreach my $p (@_)
	{
		next unless (defined $subject->property($p));
		foreach my $v ( @{$subject->property($p)->valObjects} )
		{
			push @rv, $v;
		}
	}
	return @rv;
}

sub inProp
{
	my $subject  = shift;
	return unless ($subject);
	my $look_for = shift;
	foreach my $p (@_)
	{
		next unless $subject->property($p);
		foreach my $v (@{$subject->property($p)->valObjects})
		{
			if ($v->toString eq $look_for)
			{
				return $look_for;
			}
		}
	}
	return;
}

sub findSubject
{
	my $triples = shift;
	my $url     = shift;
	
	return $triples->{$url}
		if ($triples->{$url});
	return
		unless ($url =~ /^[a-z0-9]+\:/i);
		
	foreach my $s (keys %{$triples})
	{
		my $S = $triples->{$s};
		my @v = allOf($S,
			'http://purl.org/dc/elements/1.1/identifier',
			'http://purl.org/dc/terms/identifier');
		foreach my $v (@v)
			{ return $triples->{$s} if ($v eq $url); }
	}
		
	return;
}

1;
