#!/usr/bin/perl

######################################################################
package Cognition::uF::xFolk;
######################################################################

use XML::LibXML qw(:all);
use strict;
use CGI::Util;
use Data::Dumper;
use Cognition::MagicString;
use Cognition::Export;
use Cognition::Export::Contact;

BEGIN
{
	our $VERSION = 0.1;
	our @ISA = qw(Cognition::uF);
}


sub MAX_INCLUDE_ITERATIONS { 2; }


sub consume
{
	my $page        = shift;

	if ( $page->{Profiles}->{'http://microformats.org/wiki/xfolk-profile'}
	||  !$page->{options}->{uf_strict_profiles})
	{	
		$page->{uF}->{xFolk} = [ Cognition::uF::xFolk::parse_all($page) ];
		foreach my $xf (@{ $page->{uF}->{xFolk} })
		{
			$page->mergeSubject( $xf->rdf_subject );
			$page->mergeSubject( $xf->rdf_subject_2 );
		}
		if (defined $page->{uF}->{xFolk}->[0])
			{ $page->mergeNS( $page->{uF}->{xFolk}->[0] ); }
	}	
}


sub parse_all
{
	my $page   = shift;
	my $within = shift || $page->{DOM};
	my @nodes = Cognition::HTMLParser::Utils::searchClass('xfolkentry', $within);
	my @rv;
	
	foreach my $xf (@nodes)
	{
		my $XF = parse($page, $xf);
		push @rv, $XF;
	}
	
	return @rv;
} #/sub parse_all


sub uri
{
	my $this = shift;
	my $all  = shift;
	my @rv;

	if (lc($this->{_dom}->tagName) eq 'body')
	{
		push @rv, $this->{_page}->abs_url;
	}
	elsif (length $this->{_id})
	{
		push @rv, $this->{_page}->abs_url . '#' . $this->{taggedlink};
	}
	push @rv, Cognition::HTMLParser::Utils::BNodeURI($this->{_dom}, 'Review')
		unless (@rv);
	
	return $rv[0] unless (defined $all);
	while ($all) { shift @rv; $all--; }
	return @rv;
} #/sub uri


sub rdf_subject
{
	my $this    = shift;

	my $subject = Cognition::RDFModel::Subject->new($this->uri);
	$subject->addObject($this);
	$subject->setDom($this->{_dom});
	
	my $val = Cognition::RDFModel::Value->new($this->{'taggedlink'}, 1);
	$subject->addVal('http://purl.org/dc/terms/source', $val);

	$val = Cognition::RDFModel::Value->new($this->{'description'});
	$subject->addVal('http://purl.org/dc/terms/abstract', $val);
	
	$val = Cognition::RDFModel::Value->new($this->{'html_description'});
	$val->setDatatype('http://www.w3.org/1999/02/22-rdf-syntax-ns#XMLLiteral');
	$val->{xmlns} = 'http://www.w3.org/1999/xhtml';
	$subject->addVal('http://www.purl.org/stuff/rev#text', $val);

	$val = Cognition::RDFModel::Value->new($this->{'title'});
	$subject->addVal('http://purl.org/dc/terms/title', $val);

	$val = Cognition::RDFModel::Value->new('http://www.purl.org/stuff/rev#Review', 1);
	$subject->addVal("http://www.w3.org/1999/02/22-rdf-syntax-ns#type", $val);

	$val = Cognition::RDFModel::Value->new('url');
	$subject->addVal("http://www.purl.org/stuff/rev#type", $val);

	foreach my $V (@{ $this->{'tag'} })
	{
		if (UNIVERSAL::can($V, 'uri'))
			{ $val = Cognition::RDFModel::Value->new($V->uri, 1); }
		else
			{ $val = Cognition::RDFModel::Value->new($V); }
		$subject->addVal('http://microformats.org/wiki/rel-tag#tag', $val);
	}
	
	return $subject;
	
} #/sub rdf_subject


sub rdf_subject_2
{
	my $this    = shift;
	my $subject = Cognition::RDFModel::Subject->new($this->{'taggedlink'});
	my $val = Cognition::RDFModel::Value->new($this->{'title'});
	$subject->addVal('http://www.w3.org/2000/01/rdf-schema#label', $val);
	$val = Cognition::RDFModel::Value->new($this->uri, 1);
	$subject->addVal('http://www.purl.org/stuff/rev#hasReview', $val);
	return $subject;
} #/sub rdf_subject_2


sub metadata_ns
{
	my $this = shift;
	return {
		'tag' => {
				nsurl=>'http://microformats.org/wiki/rel-tag#',
				title=>'rel-tag'
			},
		'rev' => {
				nsurl=>'http://www.purl.org/stuff/rev#',
				title=>'RDF Review Vocab'
			},
		};
} #/sub metadata_ns


sub parse
{
	my $page       = shift;
	my $rv         = { '_dom'=>shift };
	my $xfolk_node = $rv->{'_dom'}->cloneNode(1);
	
	my $id = $xfolk_node->getAttribute('id');
	$rv->{_id} = $id if (length $id);
	$rv->{_page} = $page;
		
	# Expand microformat patterns.
	my $incl_iterations = 0;
	my $replacements = 1;
	while (($incl_iterations < MAX_INCLUDE_ITERATIONS) && $replacements)
	{
		$replacements = Cognition::uF::ufp_include($xfolk_node, $page)
		              + Cognition::uF::ufp_include2($xfolk_node, $page);
		$incl_iterations++;
	}
	Cognition::uF::ufp_abbr($xfolk_node, $page);
	Cognition::uF::ufp_table($xfolk_node, $page);

	# Categories
	my @tags = Cognition::uF::RelTag::parse_all($page, $xfolk_node);
	$rv->{tag} = \@tags unless (!@tags);
	
	# Now that we have reached here, no other composite microformats have
	# any business being nested within this root element. So in the interest
	# of Microformat opacity, let's destroy them.
	Cognition::uF::destroyer($xfolk_node);	
	
	my @desc_nodes = Cognition::HTMLParser::Utils::searchClass('description',
		$xfolk_node);
	my $desc = Cognition::HTMLParser::Utils::STRINGIFY($desc_nodes[0], 'value');
	$rv->{description} = $desc if (length $desc);
	my $hdesc = Cognition::HTMLParser::Utils::XML_STRINGIFY($desc_nodes[0]);
	$rv->{html_description} = $hdesc if (length $hdesc);
	
	my @link_nodes = Cognition::HTMLParser::Utils::searchClass('taggedlink',
		$xfolk_node);
	my $title = Cognition::HTMLParser::Utils::STRINGIFY($link_nodes[0], 'value');
	$rv->{title} = $title if (length $title);
	
	if ($link_nodes[0]->localname eq 'a' || $link_nodes[0]->localname eq 'area')
		{ $rv->{taggedlink} = $page->abs_url($link_nodes[0]->getAttribute('href')); }
	elsif ($link_nodes[0]->localname eq 'img')
		{ $rv->{taggedlink} = $page->abs_url($link_nodes[0]->getAttribute('src')); }
	elsif ($link_nodes[0]->localname eq 'object')
		{ $rv->{taggedlink} = $page->abs_url($link_nodes[0]->getAttribute('data')); }

	bless $rv;
	return $rv;
	
} #/sub parse


1; #/package Cognition::uF::xFolk

